/*
 *  Copyright (C) 2007-2008 CMM Sigma Andrzej Chmielowiec <cmmsigma@cmmsigma.eu>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 3
 *  as published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 */

#include <QtGui>

#include "MainWindow.h"
#include "ConnectAction.h"
#include "VpnItem.h"
#include "VpnNewWindow.h"
#include "VpnInfoWindow.h"
#include "VpnEditWindow.h"
#include "AboutBox.h"

#ifndef STOKEN_SOFT
#include "crd_ut10xx.h"
#endif

static const int StateClear = 0;
static const int StateInit = 1;
static const int StateConf = 2;
static const int StateLock = 3;

#ifndef STOKEN_SOFT
static int
sTokenState(CRD_UT10XX_t *ut)
{
	if (ut->info.info & 0x04)
		return StateLock;
	
	else if (ut->info.info & 0x02)
		return StateConf;
	
	else if (ut->info.info & 0x01)
		return StateInit;
	
	else
		return StateClear;
}
#endif

MainWindow::MainWindow(QWidget *parent): QWidget(parent)
{
	createActions();
	createTrayIcon();
	mTrayIcon->show();
	
#ifndef STOKEN_SOFT
	setWindowTitle(tr("sToken VPN"));
#else
	setWindowTitle(tr("sToken SoftVPN"));
#endif
	
	resize(400, 300);
	
	mIsConnected = false;
	mIsInitialized = false;
	mVpn = new QProcess(this);
	connect(mVpn, SIGNAL(readyReadStandardError()), this, SLOT(vpnOutputSlot()));
	connect(mVpn, SIGNAL(readyReadStandardOutput()), this, SLOT(vpnOutputSlot()));
	connect(mVpn, SIGNAL(finished(int, QProcess::ExitStatus)), this, 
		SLOT(vpnFinishSlot(int, QProcess::ExitStatus)));
	
	mVpnActiveItem = 0;
	mVpnConnectItem = 0;
	
	QGridLayout *layout = new QGridLayout;
	QLabel *label = new QLabel();
	label->setPixmap(QPixmap(QString(":img/stoken_logo_120_60.png")));
	label->setAlignment(Qt::AlignRight);
	layout->addWidget(label, 0, 0);
	layout->addWidget(createVpnSessions(), 1, 0);
	layout->addWidget(createVpnInfo(), 2, 0);
	
	setLayout(layout);
	
	readConfFiles();
}

void
MainWindow::closeEvent(QCloseEvent *event)
{
	if (mTrayIcon->isVisible())
	{
		hide();
		event->ignore();
	}
}

void
MainWindow::vpnConnectSlot()
{
	if (!isVisible())
		show();
	
	mVpnOutput->clear();
	
	QStringList arguments;
	
	// Get PIN and check if it is correct.
	bool ok;
	QString pin = QInputDialog::getText(this, tr("Podaj haso do klucza prytwanego"),
		tr("Haso:"), QLineEdit::Password, QString(), &ok);
	
	if (!ok)
		return;
	
#ifndef STOKEN_SOFT
	if ((CRD_UT10XX_lib_init() < 0) || (CRD_UT10XX_lib_find_devices() < 0)
		|| (CRD_UT10XX_lib_actualize_devices() < 0))
	{
		QMessageBox::critical(this, tr("Bd"), tr("Nie mona zainicjowa biblioteki "
			"obsugujcej tokeny!"));
		qApp->quit();
	}
	
	QString tokenName = mVpnActiveItem->key();
	tokenName.chop(tokenName.size() - tokenName.indexOf('/'));
	
	CRD_UT10XX_t *ut;
	
	if ((ut = CRD_UT10XX_find_by_name(tokenName.toUtf8().data())) == 0)
	{
		QMessageBox::critical(this, tr("Bd"), tr("Token wspierajcy to poczenie VPN "
			"nie jest podczony!"));
		CRD_UT10XX_lib_finish();
		return;
	}
	
	if (sTokenState(ut) != StateConf)
	{
		QMessageBox::critical(this, tr("Bd"), tr("Token wspierajcy to poczenie VPN "
			"nie jest gotowy do pracy!"));
		CRD_UT10XX_lib_finish();
		return;
	}
	
	CRD_UT10XX_session_open(ut);
	if (CRD_UT10XX_cmd_try_pin(ut, pin.toUtf8().data(), CRD_UT10XX_USER_PIN_ID) < 0)
	{
		QMessageBox::critical(this, tr("Bd"), tr("Podane haso jest nieprawidowe."));
		CRD_UT10XX_session_close(ut);
		CRD_UT10XX_lib_finish();
		return;
	}
	CRD_UT10XX_session_close(ut);
	
	CRD_UT10XX_lib_finish();
#endif
	
	// Set up arguments for VPN client.
	arguments << "--config" << "conf";
	arguments << "--dev" << "tun";
	arguments << "--float";
	arguments << "--pull";
	arguments << "--tls-client";
	arguments << "--engkey";
#ifndef STOKEN_SOFT
	arguments << "--engine" << QString("stoken:log_file:log:user_pin:").append(pin);
#else
	arguments << "--engine" << QString("stokensoft:user_pin:").append(pin);
#endif
	
	mVpn->setWorkingDirectory(QDir::current().path() + QDir::separator() + mVpnActiveItem->confName());
	
	// Start VPN client.
#	ifdef Q_WS_WIN
	mVpn->start(QCoreApplication::applicationDirPath().append(QString("/../bin/stoken_openvpn.exe")), 
		arguments);
#	else
	mVpn->start(QCoreApplication::applicationDirPath().append(QString("/../bin/stoken_openvpn")), 
		arguments);
#	endif
	
	if (!mVpn->waitForStarted())
	{
		QMessageBox::critical(this, tr("Bd"), tr("Nie mona zainicjowa "
			"poczenia VPN!"));
		return;
	}
	
	mIsConnected = true;
	mConnectMenu->setEnabled(!mIsConnected);
	mDisConnectAction->setEnabled(mIsConnected);
	mConnectInfoAction->setEnabled(mIsConnected);
	
	mVpnActiveItem->setIsConnected(true);
	mVpnConnectItem = mVpnActiveItem;
	vpnItemActiveSlot(mVpnActiveItem);
	
	for (int i = 0; i < mVpnList->count(); i++)
	{
		VpnItem *item = static_cast<VpnItem*>(mVpnList->item(i));
		
		if (mVpnConnectItem == item)
			continue;
		
		item->setIsLock(true);
	}
}

void
MainWindow::vpnDisConnectSlot()
{
	if (mVpn->state() != QProcess::NotRunning)
		mVpn->kill();
}

void
MainWindow::vpnFinishSlot(int exitCode, QProcess::ExitStatus exitStatus)
{
	exitCode = 0;
	exitStatus = QProcess::NormalExit;
	
#	ifdef Q_WS_WIN
	mTrayIcon->setIcon(QIcon(QString(":img/vpn_dis_16.png")));
#	else
	mTrayIcon->setIcon(QIcon(QString(":img/vpn_dis_22.png")));
#	endif
	mTrayIcon->setToolTip(tr("sToken VPN - brak poczenia"));
	mTrayIcon->showMessage(tr("Zakoczono poczenie"), tr("Poczenie VPN zostao "
		"zakoczone."));
	
	mIsConnected = false;
	mIsInitialized = false;
	mConnectMenu->setEnabled(!mIsConnected);
	mDisConnectAction->setEnabled(mIsConnected);
	mConnectInfoAction->setEnabled(mIsConnected);
	
	mVpnConnectItem->setIsConnected(false);
	mVpnConnectItem->setIcon(QIcon(QString(":img/vpn_22.png")));
	vpnItemActiveSlot(mVpnActiveItem);
	
	for (int i = 0; i < mVpnList->count(); i++)
	{
		VpnItem *item = static_cast<VpnItem*>(mVpnList->item(i));
		
		if (mVpnConnectItem == item)
			continue;
		
		item->setIsLock(false);
	}
	
	mVpnConnectItem = 0;
}

void
MainWindow::vpnOutputSlot()
{
	mVpnOutput->append(QString(mVpn->readAll()));
	
	if (!mIsInitialized)
	{
		QString output = mVpnOutput->toPlainText();
		
		if (output.indexOf(QString("Initialization Sequence Completed")) != -1)
		{
			mIsInitialized = true;
			mVpnConnectItem->setIcon(QIcon(QString(":img/vpn_con_22.png")));
#			ifdef Q_WS_WIN
			mTrayIcon->setIcon(QIcon(QString(":img/vpn_con_16.png")));
#			else
			mTrayIcon->setIcon(QIcon(QString(":img/vpn_con_22.png")));
#			endif
			mTrayIcon->setToolTip(tr("sToken VPN - %1").arg(mVpnConnectItem->sTokenVpnName()));
			mTrayIcon->showMessage(tr("Nawizano poczenie"), tr("Poczenie VPN zostao "
				"poprawnie zainicjowane."));
		}
	}
}

void
MainWindow::vpnItemActiveSlot(QListWidgetItem *item)
{
	mVpnActiveItem = static_cast<VpnItem*>(item);
	
	mVpnInfo->setEnabled(true);
	
	if (mIsConnected)
	{
		mVpnNew->setEnabled(false);
		mVpnEdit->setEnabled(false);
		mVpnDelete->setEnabled(false);
	}
	else
	{
		mVpnNew->setEnabled(true);
		mVpnEdit->setEnabled(true);
		mVpnDelete->setEnabled(true);
	}
	
	if (mVpnActiveItem->isLock())
	{
		mVpnConnect->setEnabled(false);
		mVpnDisConnect->setEnabled(false);
	}
	else if (mVpnActiveItem->isConnected())
	{
		mVpnConnect->setEnabled(false);
		mVpnDisConnect->setEnabled(true);
	}
	else
	{
		mVpnConnect->setEnabled(true);
		mVpnDisConnect->setEnabled(false);
	}
}

void
MainWindow::vpnNewSlot()
{
	VpnNewWindow *dialog = new VpnNewWindow(this);
	dialog->exec();
	
	if (dialog->result() == QDialog::Accepted)
		readConfFiles();
	
	delete dialog;
}

void
MainWindow::vpnInfoSlot()
{
	VpnInfoWindow *dialog = new VpnInfoWindow(mVpnActiveItem, this);
	
	if (!isVisible())
		show();
	
	dialog->exec();
	delete dialog;
}

void
MainWindow::vpnEditSlot()
{
	VpnEditWindow *dialog = new VpnEditWindow(mVpnActiveItem, this);
	dialog->exec();
	
	if (dialog->result() == QDialog::Accepted)
		readConfFiles();
	
	delete dialog;
}

void
MainWindow::vpnDeleteSlot()
{
	int button = QMessageBox::question(this, tr("Pytanie"), tr("Czy jeste pewien, e "
		"chcesz usun to poczenie VPN?"), QMessageBox::Yes | QMessageBox::No, 
		QMessageBox::No);
	
	if (button == QMessageBox::No)
		return;
	
	QDir confDir(QDir::current().path() + QString("/") + mVpnActiveItem->confName());
	QStringList files(confDir.entryList(QDir::Files));
	
	for (int i = 0; i < files.size(); i++)
		confDir.remove(files[i]);
	
	QDir::current().rmdir(mVpnActiveItem->confName());
	
	readConfFiles();
}

void
MainWindow::aboutSlot()
{
	if(!isVisible())
		show();
	
	AboutBox dlg(this);
	dlg.exec();
}

void
MainWindow::trayIconActive(QSystemTrayIcon::ActivationReason reason)
{
	if (reason == QSystemTrayIcon::Trigger)
	{
		if (isVisible()) hide();
		else show();
	}
}

void
MainWindow::exit()
{
	if (mVpn->state() != QProcess::NotRunning)
		mVpn->kill();
	
	qApp->quit();
}

void
MainWindow::createActions()
{
	mOpenAction = new QAction(QIcon(QString(":img/restore_22.png")), tr("&Otwrz"), this);
	connect(mOpenAction, SIGNAL(triggered()), this, SLOT(show()));
	
	mDisConnectAction = new QAction(QIcon(QString(":img/vpn_dis_22.png")), tr("Ro&zcz"), this);
	mDisConnectAction->setEnabled(false);
	connect(mDisConnectAction, SIGNAL(triggered()), this, SLOT(vpnDisConnectSlot()));
	
	mConnectInfoAction = new QAction(tr("O poczen&iu"), this);
	mConnectInfoAction->setEnabled(false);
	connect(mConnectInfoAction, SIGNAL(triggered()), this, SLOT(vpnInfoSlot()));
	
	mAboutAction = new QAction(QIcon(QString(":img/info_22.png")), tr("O programie"), this);
	connect(mAboutAction, SIGNAL(triggered()), this, SLOT(aboutSlot()));

	mExitAction = new QAction(QIcon(QString(":img/exit_22.png")), tr("&Zakocz"), this);
	connect(mExitAction, SIGNAL(triggered()), this, SLOT(exit()));
}

QGroupBox*
MainWindow::createVpnSessions()
{
	QGroupBox *box = new QGroupBox(tr("Zdefiniowane poczenia VPN"));
	QGridLayout *layout = new QGridLayout;
	
	mVpnList = new QListWidget;
	mVpnList->setSpacing(3);
	connect(mVpnList, SIGNAL(itemClicked(QListWidgetItem*)), this, 
		 SLOT(vpnItemActiveSlot(QListWidgetItem*)));
	
	mVpnConnect = new QPushButton(tr("Po&cz"));
	mVpnConnect->setEnabled(false);
	connect(mVpnConnect, SIGNAL(clicked()), this, SLOT(vpnConnectSlot()));
	
	mVpnDisConnect = new QPushButton(tr("Ro&zcz"));
	mVpnDisConnect->setEnabled(false);
	connect(mVpnDisConnect, SIGNAL(clicked()), this, SLOT(vpnDisConnectSlot()));
	
	mVpnNew = new QPushButton(tr("&Nowe"));
	connect(mVpnNew, SIGNAL(clicked()), this, SLOT(vpnNewSlot()));
	
	mVpnInfo = new QPushButton(tr("&Informacja"));
	mVpnInfo->setEnabled(false);
	connect(mVpnInfo, SIGNAL(clicked()), this, SLOT(vpnInfoSlot()));
	
	mVpnEdit = new QPushButton(tr("&Edytuj"));
	mVpnEdit->setEnabled(false);
	connect(mVpnEdit, SIGNAL(clicked()), this, SLOT(vpnEditSlot()));
	
	mVpnDelete = new QPushButton(tr("U&su"));
	mVpnDelete->setEnabled(false);
	connect(mVpnDelete, SIGNAL(clicked()), this, SLOT(vpnDeleteSlot()));
	
	layout->addWidget(mVpnList, 0, 0, 7, 1);
	layout->addWidget(mVpnConnect, 0, 1);
	layout->addWidget(mVpnDisConnect, 1, 1);
	layout->addWidget(mVpnNew, 3, 1);
	layout->addWidget(mVpnInfo, 4, 1);
	layout->addWidget(mVpnEdit, 5, 1);
	layout->addWidget(mVpnDelete, 6, 1);
	
	layout->setRowStretch(2, 0);
	
	box->setLayout(layout);
	
	return box;
}

QGroupBox*
MainWindow::createVpnInfo()
{
	QGroupBox *box = new QGroupBox(tr("Przebieg poczenia VPN"));
	QGridLayout *layout = new QGridLayout;
	
	mVpnOutput = new QTextEdit(this);
	mVpnOutput->setReadOnly(true);
	layout->addWidget(mVpnOutput, 0, 0);
	
	box->setLayout(layout);
	
	return box;
}

void
MainWindow::readConfFiles()
{
	QStringList confDirs(QDir::current().entryList(QDir::Dirs | QDir::NoDotAndDotDot));
	
	mVpnInfo->setEnabled(false);
	mVpnEdit->setEnabled(false);
	mVpnConnect->setEnabled(false);
	mVpnDisConnect->setEnabled(false);
	mVpnDelete->setEnabled(false);
	
	mVpnList->clear();
	mConnectMenu->clear();
	
	for (int i = 0; i < confDirs.size(); i++)
	{
		VpnItem *item = new VpnItem(QDir::current().filePath(confDirs[i]).append(QString("/conf")), 
			confDirs[i]);
		ConnectAction *action = new ConnectAction(item, this);
		mVpnList->addItem(item);
		mConnectMenu->addAction(action);
		connect(action, SIGNAL(vpnItemActiveSig(QListWidgetItem*)), this, 
			SLOT(vpnItemActiveSlot(QListWidgetItem*)));
		connect(action, SIGNAL(vpnConnectSig()), this, SLOT(vpnConnectSlot()));
	}
}

void
MainWindow::createTrayIcon()
{
	mTrayIconMenu = new QMenu(this);
	mTrayIconMenu->addAction(mOpenAction);
	mTrayIconMenu->addSeparator();
	mConnectMenu = mTrayIconMenu->addMenu(QIcon(QString(":img/vpn_con_22.png")), tr("Po&cz"));
	mTrayIconMenu->addAction(mDisConnectAction);
	mTrayIconMenu->addAction(mConnectInfoAction);
	mTrayIconMenu->addSeparator();
	mTrayIconMenu->addAction(mAboutAction);
	mTrayIconMenu->addSeparator();
	mTrayIconMenu->addAction(mExitAction);
	
	mTrayIcon = new QSystemTrayIcon(this);
	mTrayIcon->setContextMenu(mTrayIconMenu);
#	ifdef Q_WS_WIN
	mTrayIcon->setIcon(QIcon(QString(":img/vpn_dis_16.png")));
#	else
	mTrayIcon->setIcon(QIcon(QString(":img/vpn_dis_22.png")));
#	endif
	mTrayIcon->setToolTip(tr("sToken VPN - brak poczenia"));
	connect(mTrayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), this, \
		SLOT(trayIconActive(QSystemTrayIcon::ActivationReason)));
	
#ifndef STOKEN_SOFT
	setWindowIcon(QIcon(QString(":img/stoken_vpn_032.png")));
#else
	setWindowIcon(QIcon(QString(":img/stoken_softvpn_032.png")));
#endif	
}
